# -*- coding: utf-8 -*-
from django.db import models, connection
from django.db.models import Avg, Sum, Min, Max
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import PermissionDenied
from django.core.mail import EmailMultiAlternatives

from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel, DefaultModel
from jumbo.middleware.thread_local import get_current_user
from jumbo.utils.functions import get_settings
from jumbo.shortcuts import render_to_string

from organization.models import Contact, Currency

######################################
# NOTE SPESE                         #
######################################
class ExpenseManager(models.Manager):
    def get_query_set(self):
        user = get_current_user()
        groups = user.groups.all()
        qs = super(ExpenseManager, self).get_query_set()

        ## se può vedere tutti le note spese ritorno il QS non filtrato
        if user.has_perm("expense.list_all_expenses"):
            return qs

        ### se può vedere i documenti PROPRI
        if user.has_perm("expense.list_own_expenses"):
            return qs.filter(assigned_to__user=user)

        return qs.none()

class Expense(DateModel, UserModel, StatusModel):
    objects = ExpenseManager()
    assigned_to = models.ForeignKey (
        ## l'intestatario della nota spese è un 'contact' quindi una 'persona' dell'azienda stessa, di un'altra azienda o autonoma
        ## non necessariamente l'intestatario della nota spese è un utente del sistema
        Contact,
        db_index = True,
        related_name='expenses',
        verbose_name = _("Assigned to"),
    )
    authorized_by_manager = models.BooleanField (
        db_index = True,
        verbose_name = _("Auth by Manager"),
    )
    authorized_by_administration = models.BooleanField (
        db_index = True,
        verbose_name = _("Auth by Administration"),
    )
    currency = models.ForeignKey (
        Currency,
        related_name = "expenses",
        db_index = True,
        verbose_name = _("Document Currency")
    )
    destination_currency = models.ForeignKey (
        Currency,
        related_name = "destination_expenses",
        db_index = True,
        blank = True, null = True,
        verbose_name = _("Destination Currency")
    )
    change = models.DecimalField (
        blank = True, null = True,
        max_digits = 9,
        decimal_places = 4
    )
    date_begin = models.DateField (
        db_index = True,
        verbose_name = _("Date From"),
    )
    date_end = models.DateField (
        db_index = True,
        verbose_name = _("Date To"),
    )
    code = models.CharField (
        max_length = 10,
        db_index = True,
        verbose_name = _("Code")
    )
    year = models.PositiveIntegerField (
        db_index = True,
        verbose_name = _("Year")
    )
    purpose_trip = models.TextField (
        ## Note
        blank = True, null = True,
        verbose_name = _("Purpose of trip"),
    )
    number = models.IntegerField (
        ## numero progressivo annuale
        db_index = True,
        verbose_name = _("Number"),
    )
    paid_on = models.DateField(blank=True, null=True,db_index=True)
    calc_transportation_total = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Taxi"),
    )
    calc_autotravel_total = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Taxi"),
    )
    calc_meal_total = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Taxi"),
    )
    calc_lodging_total = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Taxi"),
    )
    calc_client_entertainment_total = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Taxi"),
    )
    calc_private_expensive_total = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Taxi"),
    )
    calc_company_express_total = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Taxi"),
    )
    calc_other_expense_total = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Taxi"),
    )
    calc_total_expense =  models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Taxi"),
    )

    def save(self, scan_to_mail=False):
        can_be_save = False
        is_add = False

        ## INSERIMENTO nuuovo nota spese
        if not self.id:
            is_add = True
            if self.can_be_added():
                can_be_save = True

        ## MODIFICA nota spese esistente
        else:
            if self.can_be_modified():
                can_be_save = True

        ## tutte le volte che salvo setto YEAR
        self.year = self.date_begin.year

        ## VERIFICO che sia stato impostato un numero, altrimenti lo setto
        if not self.number:
            from expense.utils import set_number
            organization = self.assigned_to.organization
            self.number = set_number(self.year, organization)

        if self.currency.code == "EUR" and not self.change:
            self.change = 1

        self.calc_total_expense = self.totals

        if can_be_save:
            super(Expense, self).save()

            if is_add:
                self.send_mail_on_add_expense()
            return

        raise PermissionDenied


    def send_mail_on_add_expense(self):
        TO_SEND_MAIL_ON_ADD_EXPENSE = get_settings('expense.settings', 'TO_SEND_MAIL_ON_ADD_EXPENSE')

        if TO_SEND_MAIL_ON_ADD_EXPENSE:
            from_email = get_settings('expense.settings', 'FROM_SEND_MAIL_ON_ADD_EXPENSE')
            subject = get_settings('expense.settings', 'SUBJECT_SEND_MAIL_ON_ADD_EXPENSE')
            to = TO_SEND_MAIL_ON_ADD_EXPENSE

            text_content = render_to_string("admin/expense/expense/mail_on_add_expense.txt", expense=self)
            html_content = render_to_string("admin/expense/expense/mail_on_add_expense.html", expense=self)

            msg = EmailMultiAlternatives(subject, text_content, from_email, to)
            msg.attach_alternative(html_content, "text/html")
            msg.send()


    def can_be_added(self):
        from jumbo.middleware.thread_local import get_current_user
        user = get_current_user()
        if not self.id:
            if user.has_perm("expense.add_expense"):
                return True
            elif user.has_perm("expense.add_owner_expense") and self.assigned_to.user == user:
                return True
        return False

    def can_be_modified(self):
        from jumbo.middleware.thread_local import get_current_user
        user = get_current_user()
        if self.id:
            if user.has_perm("expense.change_expense"):
                return True
            elif user.has_perm("expense.change_owner_expense") and self.assigned_to.user == user:
                return True
        return False

    def can_be_deleted(self):
        from jumbo.middleware.thread_local import get_current_user
        user = get_current_user()
        if self.id:
            if user.has_perm("expense.delete_expense"):
                return True
            elif user.has_perm("expense.delete_owner_expense") and self.assigned_to.user == user:
                return True
        return False

    def transportation_total(self):
        air_cost_subtotal   = 0
        bus_cost_subtotal   = 0
        train_cost_subtotal = 0
        taxi_cost_subtotal  = 0
        total               = 0

        for x in self.transportations.all():
            air_cost_subtotal       += x.air_cost
            bus_cost_subtotal       += x.bus_cost
            train_cost_subtotal     += x.train_cost
            taxi_cost_subtotal      += x.taxi_cost
            total                   += x.total

        transportation_totals = {
            "air_cost_subtotal": air_cost_subtotal, "bus_cost_subtotal": bus_cost_subtotal, "train_cost_subtotal": train_cost_subtotal,
            "taxi_cost_subtotal": taxi_cost_subtotal, "total": total
        }
        return transportation_totals
    transportation_total = property(transportation_total)

    def euro_transportation_total(self):
        try:
            return self.transportation_total['total'] * self.change
        except:
            return 0
    euro_transportation_total = property(euro_transportation_total)

    def autotravel_total(self):
        km_cost_subtotal        = 0
        rental_cost_subtotal    = 0
        other_cost_subtotal     = 0
        total                   = 0
        for x in self.autotravels.all():
            km_cost_subtotal        += x.km_cost
            rental_cost_subtotal    += x.rental_cost
            other_cost_subtotal     += x.other_cost
            total                   += x.total

        autotravel_totals = {
            "km_cost_subtotal": km_cost_subtotal, "rental_cost_subtotal": rental_cost_subtotal, "other_cost_subtotal": other_cost_subtotal,
            "total": total
        }
        return autotravel_totals
    autotravel_total = property(autotravel_total)

    def euro_autotravel_total(self):
        try:
            return self.autotravel_total['total'] * self.change
        except:
            return 0
    euro_autotravel_total = property(euro_autotravel_total)

    def meal_total(self):
        breakfast_cost_subtotal = 0
        lunch_cost_subtotal     = 0
        dinner_cost_subtotal    = 0
        other_cost_subtotal     = 0
        total                   = 0
        for x in self.meals.all():
            breakfast_cost_subtotal += x.breakfast_cost
            lunch_cost_subtotal     += x.lunch_cost
            dinner_cost_subtotal    += x.dinner_cost
            other_cost_subtotal     += x.other_cost
            total += x.total

        meal_totals = {
            "breakfast_cost_subtotal": breakfast_cost_subtotal, "lunch_cost_subtotal": lunch_cost_subtotal, "dinner_cost_subtotal": dinner_cost_subtotal,
            "other_cost_subtotal": other_cost_subtotal, "total": total
        }
        return meal_totals
    meal_total = property(meal_total)

    def euro_meal_total(self):
        try:
            return self.meal_total['total'] * self.change
        except:
            return 0
    euro_meal_total = property(euro_meal_total)

    def lodging_total(self):
        lodging_cost_subtotal   = 0
        phone_cost_subtotal     = 0
        tip_cost_subtotal       = 0
        laundry_cost_subtotal   = 0
        meal_cost_subtotal     = 0
        other_cost_subtotal     = 0
        total                   = 0
        for x in self.lodgings.all():
            lodging_cost_subtotal   += x.lodging_cost
            phone_cost_subtotal     += x.phone_cost
            tip_cost_subtotal       += x.tip_cost
            laundry_cost_subtotal   += x.laundry_cost
            meal_cost_subtotal      += x.meal_cost
            other_cost_subtotal     += x.other_cost
            total += x.total

        lodging_totals = {
            "lodging_cost_subtotal": lodging_cost_subtotal, "phone_cost_subtotal": phone_cost_subtotal, "tip_cost_subtotal": tip_cost_subtotal,
            "laundry_cost_subtotal": laundry_cost_subtotal, "meal_cost_subtotal": meal_cost_subtotal,
            "other_cost_subtotal": other_cost_subtotal, "total": total
        }
        return lodging_totals
    lodging_total = property(lodging_total)

    def euro_lodging_total(self):
        try:
            return self.lodging_total['total'] * self.change
        except:
            return 0
    euro_lodging_total = property(euro_lodging_total)

    def client_entertainment_total(self):
        restaurant_cost_subtotal    = 0
        sport_cost_subtotal         = 0
        lodging_cost_subtotal       = 0
        air_cost_subtotal           = 0
        other_cost_subtotal         = 0
        total                       = 0
        for x in self.client_entertainments.all():
            restaurant_cost_subtotal    += x.restaurant_cost
            sport_cost_subtotal         += x.sport_cost
            lodging_cost_subtotal       += x.lodging_cost
            air_cost_subtotal           += x.air_cost
            other_cost_subtotal         += x.other_cost
            total                       += x.total

        client_entertainment_totals = {
            "restaurant_cost_subtotal": restaurant_cost_subtotal, "sport_cost_subtotal": sport_cost_subtotal, "lodging_cost_subtotal": lodging_cost_subtotal,
            "air_cost_subtotal": air_cost_subtotal, "other_cost_subtotal": other_cost_subtotal, "total": total
        }
        return client_entertainment_totals
    client_entertainment_total = property(client_entertainment_total)

    def euro_client_entertainment_total(self):
        try:
            return self.client_entertainment_total['total'] * self.change
        except:
            return 0
    euro_client_entertainment_total = property(euro_client_entertainment_total)


    def private_expensive_total(self):
        private_expensive_total = 0
        for x in self.privates.all():
            private_expensive_total += x.total
        return private_expensive_total
    private_expensive_total = property(private_expensive_total)


    def euro_private_expensive_total(self):
        try:
            return self.private_expensive_total * self.change
        except:
            return 0
    euro_private_expensive_total = property(euro_private_expensive_total)


    def company_express_total(self):
        company_express_total = 0
        for x in self.company_expresses.all():
            company_express_total += x.total
        return company_express_total
    company_express_total = property(company_express_total)


    def euro_company_express_total(self):
        try:
            return self.company_express_total * self.change
        except:
            return 0
    euro_company_express_total = property(euro_company_express_total)

    def other_expense_total(self):
        invoice_cost_subtotal = 0
        seminar_cost_subtotal = 0
        total = 0
        for x in self.other_expenses.all():
            invoice_cost_subtotal += x.invoice
            seminar_cost_subtotal += x.seminar_conference
            total += x.total
        other_expense_total = {
            "invoice_cost_subtotal" : invoice_cost_subtotal,
            "seminar_cost_subtotal" : seminar_cost_subtotal,
            "total" : total
        }
        return other_expense_total
    other_expense_total = property(other_expense_total)

    def euro_other_expense_total(self):
        try:
            return self.other_expense_total['total'] * self.change
        except:
            return 0
    euro_other_expense_total = property(euro_other_expense_total)


    def totals(self):
        return  self.transportation_total['total'] + self.autotravel_total['total'] + self.meal_total['total'] + \
                self.lodging_total['total'] + self.client_entertainment_total['total'] + self.private_expensive_total + \
                self.company_express_total + self.other_expense_total['total']
    totals = property(totals)


    def euro_totals(self):
        try:
            return self.totals * self.change
        except:
            return 0
    euro_totals = property(euro_totals)


    class Meta:
        verbose_name = _("Expense")
        verbose_name_plural = _("Expenses")
        permissions = (
            ("add_owner_expense",               "Can ADD Own Expense",),
            #("add_group_expense",               "Can ADD Group Expense",),

            ("change_owner_expense",            "Can CHANGE Own Expense",),
            #("change_group_expense",            "Can CHANGE Group Expense",),

            ("delete_owner_expense",            "Can DELETE Own Expense",),
            #("delete_group_expense",            "Can DELETE Group Expense",),

            ("view_expense",                    "Can VIEW Expense",),
            ("view_owner_expense",              "Can VIEW Own Expense",),
            #("view_group_expense",              "Can VIEW Group Expense",),

            ("list_all_expenses",               "Can LIST ALL Expenses"),
            #("list_group_expenses",             "Can LIST Group Expenses"),
            ("list_own_expenses",               "Can LIST Own Expenses"),

            ("authorized_as_manager",           "Can AUTH as Manager"),
            ("authorized_as_administration",    "Can AUTH as Administration"),
        )
        ordering = ('-year', '-number', '-date_begin')

    #def __unicode__(self):
        #return u"%i/%s" % (str(self.number), self.date_begin.year())

class Transportation(DateModel, UserModel, StatusModel):
    expense = models.ForeignKey (
        Expense,
        related_name = "transportations",
        db_index = True,
        verbose_name = _("Expense"),
    )
    date_begin = models.DateField (
        db_index = True,
        verbose_name = _("Date From")
    )
    date_end = models.DateField (
        db_index = True,
        verbose_name = _("Date To")
    )
    time_begin = models.TimeField (
        db_index = True,
        blank=True, null=True,
        verbose_name = _("Time From")
    )
    time_end = models.TimeField (
        db_index = True,
        blank=True, null=True,
        verbose_name = _("Time To"),
    )
    location = models.CharField(
        max_length = 200,
        blank=True, null=True,
        verbose_name = _("Location"),
    )
    air_cost = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Air Travel"),
    )
    bus_cost = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Bus/Subway"),
    )
    train_cost = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Train"),
    )
    taxi_cost = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Taxi"),
    )

    def save(self):
        super(Transportation, self).save()
        self.expense.calc_transportation_total = self.total
        self.expense.save()

    def total(self):
        return self.air_cost + self.bus_cost + self.train_cost + self.taxi_cost
    total = property(total)

    class Meta:
        verbose_name = _("Transport")
        verbose_name_plural = _("Transport")

class AutoTravel(DateModel, UserModel, StatusModel):
    expense = models.ForeignKey (
        Expense,
        related_name = "autotravels",
        db_index = True,
        verbose_name = _("Expense"),
    )
    date_begin = models.DateField (
        db_index = True,
        verbose_name = _("Date From")
    )
    date_end = models.DateField (
        db_index = True,
        verbose_name = _("Date To")
    )
    time_begin = models.TimeField (
        db_index = True,
        blank=True, null=True,
        verbose_name = _("Time From")
    )
    time_end = models.TimeField (
        db_index = True,
        blank=True, null=True,
        verbose_name = _("Time To"),
    )
    location = models.CharField(
        max_length = 200,
        blank=True, null=True,
        verbose_name = _("Location"),
    )
    car = models.CharField (
        # marca e modello
        max_length = 200,
        verbose_name = _("Personal Vehicle"),
    )
    car_km = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Car Km"),
    )
    rate_per_km = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Rate/KM"),
    )
    rental_cost = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Rental Cost"),
    )
    other_cost = models.DecimalField (
        # pedaggi, rifornimento, parcheggi
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Other Cost"),
    )

    def save(self):
        super(AutoTravel, self).save()
        self.expense.calc_autotravel_total = self.total
        self.expense.save()

    def total(self):
        return self.rate_per_km * self.car_km + self.rental_cost + self.other_cost
    total = property(total)

    def km_cost(self):
        return self.rate_per_km * self.car_km
    km_cost = property(km_cost)

    class Meta:
        verbose_name = _("Car Expense")
        verbose_name_plural = _("Car Expenses")

class Meal(DateModel, UserModel, StatusModel):
    expense = models.ForeignKey (
        Expense,
        related_name = "meals",
        db_index = True,
        verbose_name = _("Expense"),
    )
    date_begin = models.DateField (
        db_index = True,
        verbose_name = _("Date From")
    )
    date_end = models.DateField (
        db_index = True,
        verbose_name = _("Date To")
    )
    time_begin = models.TimeField (
        db_index = True,
        blank=True, null=True,
        verbose_name = _("Time From")
    )
    time_end= models.TimeField (
        db_index = True,
        blank=True, null=True,
        verbose_name = _("Time To")
    )
    location = models.CharField(
        max_length = 200,
        blank=True, null=True,
        verbose_name = _("Location")
    )
    breakfast_cost = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Breakfast"),
    )
    lunch_cost = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Lunch"),
    )
    dinner_cost = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Dinner"),
    )
    other_cost = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Other"),
    )

    def save(self):
        super(Meal, self).save()
        self.expense.calc_meal_total = self.total
        self.expense.save()

    def total(self):
        return self.breakfast_cost + self.lunch_cost + self.dinner_cost + self.other_cost
    total = property(total)

    class Meta:
        verbose_name = _("Meal")
        verbose_name_plural = _("Meals")

class Lodging(DateModel, UserModel, StatusModel):
    expense = models.ForeignKey (
        Expense,
        related_name = "lodgings",
        db_index = True,
        verbose_name = _("Expense"),
    )
    date_begin = models.DateField (
        db_index = True,
        verbose_name = _("Date From")
    )
    date_end = models.DateField (
        db_index = True,
        verbose_name = _("Date To")
    )
    time_begin = models.TimeField (
        db_index = True,
        blank=True, null=True,
        verbose_name = _("Time From")
    )
    time_end = models.TimeField (
        db_index = True,
        blank=True, null=True,
        verbose_name = _("Time To")
    )
    location = models.CharField(
        max_length = 200,
        blank=True, null=True,
        verbose_name = _("Location")
    )
    lodging_cost = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Apartment/hotel"),
    )
    phone_cost = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Phone"),
    )
    tip_cost = models.DecimalField (
        # mancia
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Tips"),
    )
    laundry_cost = models.DecimalField (
        # lavanderia
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Laundry"),
    )
    meal_cost = models.DecimalField (
        # lavanderia
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Meal"),
    )
    other_cost = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Other"),
    )

    def save(self):
        super(Lodging, self).save()
        self.expense.calc_lodging_total = self.total
        self.expense.save()

    def total(self):
        return self.lodging_cost + self.phone_cost + self.tip_cost + self.laundry_cost + self.meal_cost + self.other_cost
    total = property(total)

    class Meta:
        verbose_name = _("Lodging and Meals")
        verbose_name_plural = _("Lodging and Meals")

class ClientEntertainment(DateModel, UserModel, StatusModel):
    expense = models.ForeignKey (
        Expense,
        related_name = "client_entertainments",
        db_index = True,
        verbose_name = _("Expense"),
    )
    date_begin = models.DateField (
        db_index = True,
        verbose_name = _("Date From")
    )
    date_end = models.DateField (
        db_index = True,
        verbose_name = _("Date To")
    )
    time_begin = models.TimeField (
        db_index = True,
        blank=True, null=True,
        verbose_name = _("Time From")
    )
    time_end = models.TimeField (
        db_index = True,
        blank=True, null=True,
        verbose_name = _("Time To")
    )
    location = models.CharField(
        max_length = 200,
        blank=True, null=True,
        verbose_name = _("Location")
    )
    client_name = models.CharField(
        max_length = 200,
        blank=True, null=True,
        verbose_name = _("Client name")
    )
    client_company = models.CharField(
        max_length = 200,
        blank=True, null=True,
        verbose_name = _("Client Company")
    )
    restaurant_cost = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Restaurants"),
    )
    sport_cost = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Sports/Exibitions"),
    )
    lodging_cost = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Lodging"),
    )
    air_cost = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Airtravel"),
    )
    other_cost = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Gift/Other"),
    )

    def save(self):
        super(ClientEntertainment, self).save()
        self.expense.calc_client_entertainment_total = self.total
        self.expense.save()

    def total(self):
        return self.restaurant_cost + self.sport_cost + self.lodging_cost + self.air_cost + self.other_cost
    total = property(total)

    class Meta:
        verbose_name = _("Client Entertainment")
        verbose_name_plural = _("Client Entertainments")


class PrivateExpensive(DateModel, UserModel, StatusModel):
    expense = models.ForeignKey (
        Expense,
        related_name = "privates",
        db_index = True,
        verbose_name = _("Expense"),
    )
    date_begin = models.DateField (
        db_index = True,
        verbose_name = _("Date From")
    )
    location = models.CharField(
        max_length = 200,
        blank=True, null=True,
        verbose_name = _("Location")
    )
    type_of_expensive = models.CharField(
        max_length = 200,
        blank=True, null=True,
        verbose_name = _("Type of expense")
    )
    cost = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Cost"),
    )
    excess_as_per_policy = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Excess as per policy"),
    )

    def save(self):
        super(PrivateExpensive, self).save()
        self.expense.calc_private_expensive_total = self.total
        self.expense.save()

    def total(self):
        return self.cost + self.excess_as_per_policy
    total = property(total)

    class Meta:
        verbose_name = _("Private Expense")
        verbose_name_plural = _("Private Expenses")


class CompanyExpress(DateModel, UserModel, StatusModel):
    expense = models.ForeignKey (
        Expense,
        related_name = "company_expresses",
        db_index = True,
        verbose_name = _("Expense"),
    )
    date_begin = models.DateField (
        db_index = True,
        verbose_name = _("Date")
    )
    supplier = models.CharField(
        max_length = 200,
        blank=True, null=True,
        verbose_name = _("Supplier")
    )
    description = models.CharField(
        max_length = 200,
        blank=True, null=True,
        verbose_name = _("Description")
    )
    invoice = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Receipt/Invoice"),
    )

    def save(self):
        super(CompanyExpress, self).save()
        self.expense.calc_company_express_total = self.total
        self.expense.save()

    def total(self):
        return self.invoice or 0
    total = property(total)

    class Meta:
        verbose_name = _("Company Expense")
        verbose_name_plural = _("Company Expenses")


class OtherExpense(DateModel, UserModel, StatusModel):
    expense = models.ForeignKey (
        Expense,
        related_name = "other_expenses",
        db_index = True,
        verbose_name = _("Expense"),
    )
    date_begin = models.DateField (
        db_index = True,
        verbose_name = _("Date")
    )
    supplier = models.CharField(
        max_length = 200,
        blank=True, null=True,
        verbose_name = _("Supplier")
    )
    description = models.CharField(
        max_length = 200,
        blank=True, null=True,
        verbose_name = _("Description")
    )
    invoice = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Receipt/Invoice"),
    )
    seminar_conference = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Seminars/Conferences"),
    )

    def save(self):
        super(OtherExpense, self).save()
        self.expense.calc_other_expense_total = self.total
        self.expense.save()

    def total(self):
        return self.invoice + self.seminar_conference
    total = property(total)

    class Meta:
        verbose_name = _("Other Expense")
        verbose_name_plural = _("Other Expenses")